/* 
 * i-scream central monitoring system
 * http://www.i-scream.org
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.conient;

//---IMPORTS---
import uk.org.iscream.cms.util.*;
import uk.org.iscream.cms.conient.datacomponents.*;
import javax.swing.*;
import javax.swing.border.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;

/**
 * This class allows a window to be shown which displays
 * internel server queue information as sent by the i-scream
 * server.
 *
 * This allows a check to be made on the current performance of
 * the internal queues.
 *
 * @author  $Author: tdb $
 * @version $Id: QueueFrame.java,v 1.21 2004/08/01 10:40:07 tdb Exp $
 */
public class QueueFrame extends JFrame {

//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public final String REVISION = "$Revision: 1.21 $";
    
//---STATIC METHODS---
    
//---CONSTRUCTORS---

    /**
     * Constructs a new frame
     */
    public QueueFrame() {
        super("Server Queue Status");
        JScrollPane scrollPane = new JScrollPane(_content,JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED , JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        getContentPane().add(scrollPane, "Center");
        JButton close = new JButton("Close Window");
        close.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                setVisible(false);
            }
        });
        getContentPane().add(close, "South");
        setIconImage((new ImageIcon(getClass().getResource("/resources/server.gif"))).getImage());
        pack();
        setVisible(true);
    }
                  
//---PUBLIC METHODS---

    /**
     * Process an incoming packet, updates the components.
     *
     * @param packet the packet to process
     *
     * @return if the proceesing was successful
     */
    public boolean update(XMLPacket packet) {
        // iterate over the packets data
        boolean displaySucessful = true;
        String queueHashCode = packet.getParam("packet.attributes.hashCode");

        // if there are no components looking after this data
        // create a new StringDataComponent for it
        if(!_queues.containsKey(queueHashCode)) {
            addQueue(packet);
        }

        // try and update the component, if it doesn't like what it gets
        // warn the user and set that this was an unsucessful display
        try {
            updateQueueInformation(packet);
        } catch (DataFormatException e) {
            Conient.addMessage("WARNING{queue frame}: " + e.getMessage());
            displaySucessful = false;
        }
        return displaySucessful;
    }

//---PRIVATE METHODS---

    /**
     * Constructs and adds a new Queue panel to the
     * current window.  This happens when a new queue packet 
     * is detected
     *
     * @param packet the packet to obtain the queue data from
     */
    private void addQueue(XMLPacket packet) {
        HashMap newQueue = new HashMap();
        String queueName = packet.getParam("packet.attributes.name");
        String queueHashCode = packet.getParam("packet.attributes.hashCode");
        VisibleDataComponent date = new DateDataComponent("Last timestamp", "packet.attributes.date");
        VisibleDataComponent total = new StringDataComponent("Total queued to date", "packet.queue.attributes.total");
        VisibleDataComponent max = new StringDataComponent("Maximum queue size", "packet.queue.attributes.maxSize");
        newQueue.put("packet.attributes.name", new StringDataComponent("Name", "packet.attributes.name"));
        newQueue.put("packet.queue.attributes.total", total);
        newQueue.put("packet.queue.attributes.maxSize", max);
        newQueue.put("packet.attributes.date", date);
        
        JPanel newQueueDisplay = new JPanel();
        newQueueDisplay.setLayout(new BoxLayout(newQueueDisplay, BoxLayout.Y_AXIS));
        newQueueDisplay.setBorder(new TitledBorder(new LineBorder(new Color(0, 0, 102)), queueName));
        newQueueDisplay.add(date);
        newQueueDisplay.add(total);
        newQueueDisplay.add(max);
        _queuesDisplays.put(queueHashCode, newQueueDisplay);
        try {
            SwingSafeAdd task = new SwingSafeAdd(_content, newQueueDisplay);
            SwingUtilities.invokeAndWait(task);
        } catch (Exception e) {
            // don't care
        }
        pack();
        _queues.put(queueHashCode, newQueue);
    }
    
    /**
     * Updates a queue panel with new data
     *
     * @param packet the packet to obtain the new data from
     * @throws DataFormatException if the packet contains invalid data
     */
    private void updateQueueInformation(XMLPacket packet) throws DataFormatException {
        String queueHashCode = packet.getParam("packet.attributes.hashCode");
        
        // if it was a shutdown packet, remove the display
        if (packet.getParam("packet.attributes.shutdown") != null) {
            // *** !!! MAY NOT BE SWING SAFE !!! ***
            JPanel temp = (JPanel)_queuesDisplays.get(queueHashCode);
            temp.setVisible(false);
            _content.remove(temp);
            _queuesDisplays.remove(temp);
            pack();

        // else update the display
        } else {
            HashMap components = (HashMap) _queues.get(queueHashCode);
            Set packetSet = packet.getSet();
            Iterator i = packetSet.iterator();
            while (i.hasNext()) {
                String dataKey = (String) i.next();
                // if there are no components looking after this data
                // create a new StringDataComponent for it
                if(!components.containsKey(dataKey)) {
                    // check if its a queue, if it is then we need to deal with it
                    if(dataKey.startsWith("packet.queue.attributes.queue")) {
                        VisibleDataComponent queue = new StringDataComponent("Current length of " + dataKey.substring(dataKey.lastIndexOf('.') + 1), dataKey);
                        components.put(dataKey, queue);
                        try {
                            SwingSafeAdd task = new SwingSafeAdd((JPanel)_queuesDisplays.get(queueHashCode), queue);
                            SwingUtilities.invokeAndWait(task);
                        } catch (Exception e) {
                            // if there was a problem
                            System.err.println("ERROR Failed To Add Component - " + e);
                        }
                        pack();
                    }
                    // note we ignore all other attributes
                }
                if(components.containsKey(dataKey)) {
                    ((DataComponent) components.get(dataKey)).setValue(packet);
                }
            }
        }
    }

//---ACCESSOR/MUTATOR METHODS---

//---ATTRIBUTES---

    /**
     * The components that we already know about
     * connected with the attributes they care about
     * are stored here.
     */
    private HashMap _queues = new HashMap();
    
    /**
     * The Panels on the display hooked to the queue hashid
     */
    private HashMap _queuesDisplays = new HashMap();
    
    /**
     * The box to lay queue panels into.
     */
    private Box _content = Box.createVerticalBox();

//---STATIC ATTRIBUTES---

}
